CIRRO_AUTH_CLIENT_ID = 'CIRRO_AUTH_CLIENT_ID'
CIRRO_DB_URI = 'CIRRO_DB_URI'
CIRRO_EMAIL = 'CIRRO_EMAIL'
CIRRO_SERVE = 'CIRRO_SERVE'
CIRRO_MAX_WORKERS = 'CIRRO_MAX_WORKERS'
CIRRO_FOOTER = 'CIRRO_FOOTER'
CIRRO_BRAND = 'CIRRO_BRAND'
CIRRO_UPLOAD = 'CIRRO_UPLOAD'
CIRRO_DATABASE_CLASS = 'CIRRO_DATABASE'

SERVER_CAPABILITY_RENAME_CATEGORIES = 'rename_categories'
SERVER_CAPABILITY_JOBS = 'jobs'
SERVER_CAPABILITY_SAVE_FEATURE_SETS = 'save_feature_sets'
SERVER_CAPABILITY_SAVE_LINKS = 'save_links'
SERVER_CAPABILITY_EDIT_DATASET = 'edit_dataset'
SERVER_CAPABILITY_ADD_DATASET = 'add_dataset'
SERVER_CAPABILITY_DELETE_DATASET = 'delete_dataset'
